@@ -16,9 +16,20 @@
 #define KRDCCORE_EXPORT
 #endif
 
+#include <QClipboard>
+#include <QMap>
 #include <QUrl>
 #include <QWidget>
 
+#ifdef HAVE_WAYLAND
+#include "shortcutinhibition_p.h"
+#endif
+
+struct ModifierKey {
+    quint32 nativeScanCode;
+    quint32 nativeVirtualKey;
+};
+
 class HostPreferences;
 
 /**
@@ -318,6 +329,19 @@ public Q_SLOTS:
      */
     virtual void scaleResize(int w, int h);
 
+    /**
+     * Internal handling of local clipboard change
+     * Do not override in subclasses; implement handleLocalClipboardChanged() instead
+     */
+    void localClipboardChanged();
+
+    /**
+     * Internal handling of remote clipboard change
+     * Subclasses must use this method to update local clipboard contents
+     * Ownership of the data is transferred to the clipboard.
+     */
+    void remoteClipboardChanged(QMimeData *data);
+
 Q_SIGNALS:
     /**
      * Emitted when the size of the remote screen changes. Also
@@ -373,7 +397,12 @@ protected:
 
     void focusInEvent(QFocusEvent *event) override;
     void focusOutEvent(QFocusEvent *event) override;
-
+    bool event(QEvent *event) override;
+    bool eventFilter(QObject *obj, QEvent *event) override;
+    virtual void handleKeyEvent(QKeyEvent *event) = 0;
+    virtual void handleWheelEvent(QWheelEvent *event) = 0;
+    virtual void handleMouseEvent(QMouseEvent *event) = 0;
+    virtual void handleLocalClipboardChanged(const QMimeData *data) = 0;
     /**
      * The status of the remote view.
      */
@@ -395,6 +424,10 @@ protected:
 
     QCursor localDefaultCursor() const;
 
+    void unpressModifiers();
+    void grabKeyboard();
+    void releaseKeyboard();
+
     QString m_host;
     int m_port;
     bool m_viewOnly;
@@ -403,12 +436,19 @@ protected:
     bool m_keyboardIsGrabbed;
     QUrl m_url;
     qreal m_factor;
+    QClipboard *m_clipboard;
+    QMap<int, ModifierKey> m_modifiers;
+#ifdef HAVE_WAYLAND
+    std::unique_ptr<ShortcutInhibition> m_inhibition;
+#endif
 
 #ifndef QTONLY
     QString readWalletPassword(bool fromUserNameOnly = false);
     void saveWalletPassword(const QString &password, bool fromUserNameOnly = false);
+    void deleteWalletPassword(bool fromUserNameOnly = false);
     QString readWalletPasswordForKey(const QString &key);
     void saveWalletPasswordForKey(const QString &key, const QString &password);
+    void deleteWalletPasswordForKey(const QString &key);
     KWallet::Wallet *m_wallet;
 #endif
 
