@@ -133,6 +133,9 @@ QWidget *RdpHostPreferences::createProtocolSpecificConfigPage()
     rdpUi.kcfg_Resolution->setCurrentIndex(int(resolution()));
     rdpUi.kcfg_Acceleration->setCurrentIndex(int(acceleration()));
     rdpUi.kcfg_ColorDepth->setCurrentIndex(int(colorDepth()));
+    rdpUi.kcfg_DesktopScaleFactor->setCurrentIndex(int(desktopScaleFactor()));
+    rdpUi.kcfg_DesktopScaleFactorCustom->setValue(desktopScaleFactorCustom());
+    rdpUi.kcfg_DeviceScaleFactor->setCurrentIndex(int(deviceScaleFactor()));
     rdpUi.kcfg_KeyboardLayout->setCurrentIndex(keymap2int(keyboardLayout()));
     rdpUi.kcfg_ShareMedia->setText(shareMedia());
     rdpUi.kcfg_TlsSecLevel->setCurrentIndex(int(tlsSecLevel()));
@@ -145,6 +148,12 @@ QWidget *RdpHostPreferences::createProtocolSpecificConfigPage()
     rdpUi.kcfg_GatewayPassword->setPassword(gatewayPassword());
     rdpUi.kcfg_GatewayServer->setText(gatewayServer());
     rdpUi.kcfg_GatewayDomain->setText(gatewayDomain());
+    rdpUi.kcfg_SmartcardName->setText(smartcardName());
+    rdpUi.kcfg_NLASecurity->setChecked(securityNLA());
+    rdpUi.kcfg_TLSSecurity->setChecked(securityTLS());
+    rdpUi.kcfg_RDPSecurity->setChecked(securityRDP());
+    rdpUi.kcfg_EXTSecurity->setChecked(securityEXT());
+    rdpUi.kcfg_AuthPkgList->setText(authPkgList());
 
     // Have to call updateWidthHeight() here
     // We leverage the final part of this function to enable/disable kcfg_Height and kcfg_Width
@@ -161,6 +170,12 @@ QWidget *RdpHostPreferences::createProtocolSpecificConfigPage()
         updateColorDepth(Acceleration(index));
     });
 
+    // kcfg_DesktopScaleFactorCustom is enabled only when kcfg_DesktopScaleFactor = DesktopScaleFactor::Custom
+    updateDesktopScaleFactor(desktopScaleFactor());
+    connect(rdpUi.kcfg_DesktopScaleFactor, &QComboBox::currentIndexChanged, this, [this](int index) {
+        updateDesktopScaleFactor(DesktopScaleFactor(index));
+    });
+
     return rdpPage;
 }
 
@@ -185,8 +200,8 @@ void RdpHostPreferences::updateWidthHeight(Resolution resolution)
             window = window->parentWidget();
         }
         if (window) {
-            rdpUi.kcfg_Width->setValue(window->width());
-            rdpUi.kcfg_Height->setValue(window->height());
+            rdpUi.kcfg_Width->setValue(window->width() * window->devicePixelRatio());
+            rdpUi.kcfg_Height->setValue(window->height() * window->devicePixelRatio());
         }
         break;
     }
@@ -222,6 +237,9 @@ void RdpHostPreferences::acceptConfig()
     setResolution(Resolution(rdpUi.kcfg_Resolution->currentIndex()));
     setAcceleration(Acceleration(rdpUi.kcfg_Acceleration->currentIndex()));
     setColorDepth(ColorDepth(rdpUi.kcfg_ColorDepth->currentIndex()));
+    setDesktopScaleFactor(DesktopScaleFactor(rdpUi.kcfg_DesktopScaleFactor->currentIndex()));
+    setDesktopScaleFactorCustom(rdpUi.kcfg_DesktopScaleFactorCustom->value());
+    setDeviceScaleFactor(DeviceScaleFactor(rdpUi.kcfg_DeviceScaleFactor->currentIndex()));
     setKeyboardLayout(int2keymap(rdpUi.kcfg_KeyboardLayout->currentIndex()));
     setSound(Sound(rdpUi.kcfg_Sound->currentIndex()));
     setShareMedia(rdpUi.kcfg_ShareMedia->text());
@@ -235,6 +253,12 @@ void RdpHostPreferences::acceptConfig()
     setGatewayPassword(rdpUi.kcfg_GatewayPassword->password());
     setGatewayServer(rdpUi.kcfg_GatewayServer->text());
     setGatewayDomain(rdpUi.kcfg_GatewayDomain->text());
+    setSmartcardName(rdpUi.kcfg_SmartcardName->text());
+    setSecurityNLA(rdpUi.kcfg_NLASecurity->isChecked());
+    setSecurityTLS(rdpUi.kcfg_TLSSecurity->isChecked());
+    setSecurityRDP(rdpUi.kcfg_RDPSecurity->isChecked());
+    setSecurityEXT(rdpUi.kcfg_EXTSecurity->isChecked());
+    setAuthPkgList(rdpUi.kcfg_AuthPkgList->text());
 }
 
 bool RdpHostPreferences::scaleToSize() const
@@ -422,3 +446,92 @@ void RdpHostPreferences::setGatewayDomain(const QString &gatewayDomain)
 {
     m_configGroup.writeEntry("gatewayDomain", gatewayDomain);
 }
+
+QString RdpHostPreferences::smartcardName() const
+{
+    return QString(m_configGroup.readEntry("smartcardName", Settings::smartcardName()));
+}
+void RdpHostPreferences::setSmartcardName(const QString &smartcardName)
+{
+    m_configGroup.writeEntry("smartcardName", smartcardName);
+}
+
+bool RdpHostPreferences::securityNLA() const
+{
+    return m_configGroup.readEntry("securityNLA", Settings::securityNLA());
+}
+void RdpHostPreferences::setSecurityNLA(bool enabled)
+{
+    m_configGroup.writeEntry("securityNLA", enabled);
+}
+bool RdpHostPreferences::securityTLS() const
+{
+    return m_configGroup.readEntry("securityTLS", Settings::securityTLS());
+}
+void RdpHostPreferences::setSecurityTLS(bool enabled)
+{
+    m_configGroup.writeEntry("securityTLS", enabled);
+}
+bool RdpHostPreferences::securityRDP() const
+{
+    return m_configGroup.readEntry("securityRDP", Settings::securityRDP());
+}
+void RdpHostPreferences::setSecurityRDP(bool enabled)
+{
+    m_configGroup.writeEntry("securityRDP", enabled);
+}
+bool RdpHostPreferences::securityEXT() const
+{
+    return m_configGroup.readEntry("securityEXT", Settings::securityEXT());
+}
+void RdpHostPreferences::setSecurityEXT(bool enabled)
+{
+    m_configGroup.writeEntry("securityEXT", enabled);
+}
+
+QString RdpHostPreferences::authPkgList() const
+{
+    return m_configGroup.readEntry("authPkgList", Settings::authPkgList());
+}
+void RdpHostPreferences::setAuthPkgList(const QString &authPkgList)
+{
+    m_configGroup.writeEntry("authPkgList", authPkgList);
+}
+
+void RdpHostPreferences::updateDesktopScaleFactor(DesktopScaleFactor desktopScaleFactor)
+{
+    switch (desktopScaleFactor) {
+    case DesktopScaleFactor::Auto:
+    case DesktopScaleFactor::DoNotScale:
+        rdpUi.kcfg_DesktopScaleFactorCustom->setEnabled(false);
+        break;
+    case DesktopScaleFactor::Custom:
+        rdpUi.kcfg_DesktopScaleFactorCustom->setEnabled(true);
+        break;
+    }
+}
+
+RdpHostPreferences::DesktopScaleFactor RdpHostPreferences::desktopScaleFactor() const
+{
+    return DesktopScaleFactor(m_configGroup.readEntry("desktopScaleFactor", Settings::desktopScaleFactor()));
+}
+void RdpHostPreferences::setDesktopScaleFactor(DesktopScaleFactor desktopScaleFactor)
+{
+    m_configGroup.writeEntry("desktopScaleFactor", int(desktopScaleFactor));
+}
+int RdpHostPreferences::desktopScaleFactorCustom() const
+{
+    return m_configGroup.readEntry("desktopScaleFactorCustom", Settings::desktopScaleFactorCustom());
+}
+void RdpHostPreferences::setDesktopScaleFactorCustom(int desktopScaleFactorCustom)
+{
+    m_configGroup.writeEntry("desktopScaleFactorCustom", desktopScaleFactorCustom);
+}
+RdpHostPreferences::DeviceScaleFactor RdpHostPreferences::deviceScaleFactor() const
+{
+    return DeviceScaleFactor(m_configGroup.readEntry("deviceScaleFactor", Settings::deviceScaleFactor()));
+}
+void RdpHostPreferences::setDeviceScaleFactor(DeviceScaleFactor deviceScaleFactor)
+{
+    m_configGroup.writeEntry("deviceScaleFactor", int(deviceScaleFactor));
+}
