@@ -12,10 +12,12 @@
 #include "rdphostpreferences.h"
 
 #include <KMessageBox>
+#include <KMessageDialog>
 #include <KPasswordDialog>
 #include <KShell>
 #include <KWindowSystem>
 
+#include <QChar>
 #include <QDir>
 #include <QEvent>
 #include <QInputDialog>
@@ -23,13 +25,15 @@
 #include <QMouseEvent>
 #include <QPainter>
 #include <QScreen>
+#include <QUrlQuery>
 #include <QWindow>
 
 #include "rdpsession.h"
 
-RdpView::RdpView(QWidget *parent, const QUrl &url, KConfigGroup configGroup, const QString &user, const QString &password)
+RdpView::RdpView(QWidget *parent, const QUrl &url, KConfigGroup configGroup, const QString &user, const QString &domain, const QString &password)
     : RemoteView(parent)
     , m_user(user)
+    , m_domain(domain)
     , m_password(password)
 {
     m_url = url;
@@ -40,6 +44,23 @@ RdpView::RdpView(QWidget *parent, const QUrl &url, KConfigGroup configGroup, con
         m_user = m_url.userName();
     }
 
+    if (m_domain.isEmpty()) {
+        if (m_url.hasQuery()) {
+            QUrlQuery query(m_url);
+            QString queryDomain = query.queryItemValue(QStringLiteral("domain"));
+            if (!queryDomain.isEmpty()) {
+                m_domain = queryDomain;
+            }
+        } else {
+            // convert legacy DOMAIN\\user@host URLs
+            QStringList splitted = m_user.split(QChar::fromLatin1('\\'));
+            if (splitted.size() == 2) {
+                m_domain = splitted[0];
+                m_user = splitted[1];
+            }
+        }
+    }
+
     if (m_password.isEmpty() && !m_url.password().isEmpty()) {
         m_password = m_url.password();
     }
@@ -71,6 +92,10 @@ void RdpView::scaleResize(int w, int h)
 
     // handle window resizes
     resize(sizeHint());
+
+    if (m_session) {
+        m_session->sendResizeEvent(QSize(w, h) * devicePixelRatio());
+    }
 }
 
 QSize RdpView::sizeHint() const
@@ -116,6 +141,8 @@ bool RdpView::start()
     m_session->setHost(m_host);
     m_session->setPort(m_port);
     m_session->setUser(m_user);
+    m_session->setDomain(m_domain);
+
     m_session->setSize(initialSize());
 
     if (m_password.isEmpty()) {
@@ -149,6 +176,12 @@ bool RdpView::start()
         }
     });
     connect(m_session.get(), &RdpSession::errorMessage, this, &RdpView::handleError);
+    connect(m_session.get(), &RdpSession::onLogonError, this, &RdpView::onLogonError);
+    connect(m_session.get(), &RdpSession::onAuthRequested, this, &RdpView::onAuthRequested, Qt::BlockingQueuedConnection);
+    connect(m_session.get(), &RdpSession::onVerifyCertificate, this, &RdpView::onVerifyCertificate, Qt::BlockingQueuedConnection);
+    connect(m_session.get(), &RdpSession::onVerifyChangedCertificate, this, &RdpView::onVerifyChangedCertificate, Qt::BlockingQueuedConnection);
+
+    connect(m_session.get(), &RdpSession::cursorChanged, this, &RdpView::setRemoteCursor);
 
     setStatus(RdpView::Connecting);
     if (!m_session->start()) {
@@ -161,11 +194,100 @@ bool RdpView::start()
     return true;
 }
 
+void RdpView::onAuthRequested()
+{
+    std::unique_ptr<KPasswordDialog> dialog;
+    dialog =
+        std::make_unique<KPasswordDialog>(nullptr, KPasswordDialog::ShowUsernameLine | KPasswordDialog::ShowKeepPassword | KPasswordDialog::ShowDomainLine);
+    dialog->setPrompt(i18nc("@label", "Access to this system requires a username and password."));
+    dialog->setUsername(m_user);
+    dialog->setDomain(m_domain);
+    dialog->setPassword(m_password);
+
+    if (!dialog->exec()) {
+        return;
+    }
+
+    m_user = dialog->username();
+    m_domain = dialog->domain();
+    m_password = dialog->password();
+
+    // update m_url so it gets saved correctly
+    m_url.setUserName(m_user);
+    QUrlQuery query(m_url);
+    query.removeQueryItem(QStringLiteral("domain"));
+    if (!m_domain.isEmpty()) {
+        query.addQueryItem(QStringLiteral("domain"), m_domain);
+    }
+    m_url.setQuery(query);
+
+    if (dialog->keepPassword()) {
+        savePassword(m_password);
+    }
+
+    m_session->setUser(m_user);
+    m_session->setDomain(m_domain);
+    m_session->setPassword(m_password);
+}
+
+void RdpView::onVerifyCertificate(RdpSession::CertificateResult *ret, const QString &certificate)
+{
+    KMessageDialog dialog{KMessageDialog::WarningContinueCancel, i18nc("@label", "The certificate for this system is unknown. Do you wish to continue?")};
+    dialog.setCaption(i18nc("@title:dialog", "Verify Certificate"));
+    dialog.setIcon(QIcon::fromTheme(QStringLiteral("view-certficate")));
+
+    dialog.setDetails(certificate);
+
+    dialog.setDontAskAgainText(i18nc("@label", "Remember this certificate"));
+
+    dialog.setButtons(KStandardGuiItem::cont(), KGuiItem(), KStandardGuiItem::cancel());
+
+    const auto result = static_cast<KMessageDialog::ButtonType>(dialog.exec());
+    if (result == KMessageDialog::Cancel) {
+        *ret = RdpSession::CertificateResult::DoNotAccept;
+        return;
+    }
+
+    if (dialog.isDontAskAgainChecked()) {
+        *ret = RdpSession::CertificateResult::AcceptPermanently;
+    } else {
+        *ret = RdpSession::CertificateResult::AcceptTemporarily;
+    }
+}
+
+void RdpView::onVerifyChangedCertificate(RdpSession::CertificateResult *ret, const QString &oldCertificate, const QString &newCertificate)
+{
+    KMessageDialog dialog{KMessageDialog::WarningContinueCancel, i18nc("@label", "The certificate for this system has changed. Do you wish to continue?")};
+    dialog.setCaption(i18nc("@title:dialog", "Certificate has Changed"));
+    dialog.setIcon(QIcon::fromTheme(QStringLiteral("view-certficate")));
+
+    dialog.setDetails(i18nc("@label", "Previous certificate:\n%1\nNew Certificate:\n%2", oldCertificate, newCertificate));
+
+    dialog.setDontAskAgainText(i18nc("@label", "Remember this certificate"));
+
+    dialog.setButtons(KStandardGuiItem::cont(), KGuiItem(), KStandardGuiItem::cancel());
+
+    const auto result = static_cast<KMessageDialog::ButtonType>(dialog.exec());
+    if (result == KMessageDialog::Cancel) {
+        *ret = RdpSession::CertificateResult::DoNotAccept;
+        return;
+    }
+
+    if (dialog.isDontAskAgainChecked()) {
+        *ret = RdpSession::CertificateResult::AcceptPermanently;
+    } else {
+        *ret = RdpSession::CertificateResult::AcceptTemporarily;
+    }
+}
+
 void RdpView::handleError(const unsigned int error)
 {
     QString title;
     QString message;
 
+    if (m_quitting)
+        return;
+
     switch (error) {
     case FREERDP_ERROR_CONNECT_CANCELLED:
         return; // user canceled connection, no need to show an error message
@@ -240,6 +362,11 @@ void RdpView::handleError(const unsigned int error)
     startQuitting();
 }
 
+void RdpView::onLogonError(const QString &error)
+{
+    KMessageBox::error(this, error, i18nc("@title:dialog", "Logon Error"));
+}
+
 HostPreferences *RdpView::hostPreferences()
 {
     return m_hostPreferences.get();
@@ -247,7 +374,7 @@ HostPreferences *RdpView::hostPreferences()
 
 QPixmap RdpView::takeScreenshot()
 {
-    if (!m_session->videoBuffer()->isNull()) {
+    if (m_session && !m_session->videoBuffer()->isNull()) {
         return QPixmap::fromImage(*m_session->videoBuffer());
     }
     return QPixmap{};
@@ -258,6 +385,37 @@ bool RdpView::supportsScaling() const
     return true;
 }
 
+bool RdpView::supportsLocalCursor() const
+{
+    return true;
+}
+
+bool RdpView::supportsViewOnly() const
+{
+    return true;
+}
+
+void RdpView::showLocalCursor(LocalCursorState state)
+{
+    RemoteView::showLocalCursor(state);
+
+    if (state == CursorOn) {
+        // show local cursor, hide remote one
+        setCursor(localDefaultCursor());
+    } else {
+        // hide local cursor, show remote one
+        setCursor(m_remoteCursor);
+    }
+}
+
+void RdpView::setRemoteCursor(const QCursor cursor)
+{
+    m_remoteCursor = cursor;
+    if (m_localCursorState != CursorOn) {
+        setCursor(m_remoteCursor);
+    }
+}
+
 bool RdpView::scaling() const
 {
     return m_hostPreferences->scaleToSize();
@@ -271,10 +429,6 @@ void RdpView::enableScaling(bool scale)
     update();
 }
 
-void RdpView::setScaleFactor(float factor)
-{
-}
-
 QSize RdpView::initialSize()
 {
     switch (m_hostPreferences->resolution()) {
@@ -284,15 +438,14 @@ QSize RdpView::initialSize()
         return QSize{1600, 900};
     case RdpHostPreferences::Resolution::Large:
         return QSize{1920, 1080};
-    case RdpHostPreferences::Resolution::MatchWindow:
-        return parentWidget()->size();
     case RdpHostPreferences::Resolution::MatchScreen:
         return window()->windowHandle()->screen()->size();
     case RdpHostPreferences::Resolution::Custom:
         return QSize{m_hostPreferences->width(), m_hostPreferences->height()};
+    case RdpHostPreferences::Resolution::MatchWindow:
+    default:
+        return parentWidget()->size() * devicePixelRatio();
     }
-
-    return parentWidget()->size();
 }
 
 void RdpView::savePassword(const QString &password)
@@ -302,7 +455,7 @@ void RdpView::savePassword(const QString &password)
 
 void RdpView::paintEvent(QPaintEvent *event)
 {
-    if (m_session->videoBuffer()->isNull()) {
+    if (!m_session || m_session->videoBuffer()->isNull()) {
         return;
     }
 
@@ -324,17 +477,23 @@ void RdpView::paintEvent(QPaintEvent *event)
 
 void RdpView::handleKeyEvent(QKeyEvent *event)
 {
-    m_session->sendEvent(event, this);
+    if (m_session) {
+        m_session->sendEvent(event, this);
+    }
 }
 
 void RdpView::handleMouseEvent(QMouseEvent *event)
 {
-    m_session->sendEvent(event, this);
+    if (m_session) {
+        m_session->sendEvent(event, this);
+    }
 }
 
 void RdpView::handleWheelEvent(QWheelEvent *event)
 {
-    m_session->sendEvent(event, this);
+    if (m_session) {
+        m_session->sendEvent(event, this);
+    }
 }
 
 void RdpView::onRectangleUpdated(const QRect &rect)
@@ -345,5 +504,7 @@ void RdpView::onRectangleUpdated(const QRect &rect)
 
 void RdpView::handleLocalClipboardChanged(const QMimeData *data)
 {
-    m_session->sendClipboard(data);
+    if (m_session) {
+        m_session->sendClipboard(data);
+    }
 }
\ No newline at end of file
